import logging

from zope.app.pagetemplate import ViewPageTemplateFile
from zope.formlib.form import EditForm as BaseEditForm
from zope.formlib.form import Fields

from largeblue.tag.interfaces import ITag


class EditForm(BaseEditForm):
    form_fields = Fields(ITag).select('tagstring')




from zope.interface import Interface
from zope.annotation.interfaces import IAnnotatable
from zope.schema import List, TextLine

from zope.i18nmessageid import MessageFactory
_ = MessageFactory("largeblue")


class ITag(Interface):
    tags = List(
        title = _(u'Tags'),
        description = _(u"List of keywords or 'tags'"),
        required = False
    )
    tagstring = TextLine(
        title = _(u'Tag String'),
        description = _(u"All tags as a ' ' seperated string"),
        required = False
    )
    def add_tag(tag):
        """Add a tag"""
    
    def remove_tag(tag):
        """Remove a tag"""
    
    def update_tags(self, tags):
        """Overwrite tags with a new list of tags"""
    


class ITaggable(IAnnotatable):
    """Marker interface"""



import os.path
from zope.testing import doctest
from zope.app.testing import functional

ftesting_zcml = os.path.join(os.path.dirname(__file__), 'ftesting.zcml')
FunctionalLayer = functional.ZCMLLayer(ftesting_zcml, __name__,
                                       'FunctionalLayer')

def FunctionalDocTestSuite(module=None, **kw):
    module = doctest._normalize_module(module)
    suite = functional.FunctionalDocTestSuite(module, **kw)
    suite.layer = FunctionalLayer
    return suite

def FunctionalDocFileSuite(path, **kw):
    suite = functional.FunctionalDocFileSuite(path, **kw)
    suite.layer = FunctionalLayer
    return suite

class FunctionalTestCase(functional.FunctionalTestCase):
    layer = FunctionalLayer



from persistent.list import PersistentList
from zope.annotation.interfaces import IAnnotations
from zope.component import adapts
from zope.interface import implements

from interfaces import ITag, ITaggable

KEY = 'largeblue.tag.Tags'


class Tag(object):
    implements(ITag)
    adapts(ITaggable)
    def __init__(self, context):
        self.context = self.__parent__ = context
        annotations = IAnnotations(self.context)
        mapping = annotations.get(KEY)
        if mapping is None:
            mapping = annotations[KEY] = PersistentList([])
        self.mapping = mapping
    
    def add_tag(self, tag):
        if tag not in self.mapping:
            self.mapping.append(str(tag).encode('utf-8'))
        
    
    def remove_tag(self, tag):
        if tag in self.mapping:
            self.mapping.remove(tag)
        
    
    def update_tags(self, tags):
        # remove tags
        for tag in self.mapping:
            if tag not in tags:
                self.remove_tag(tag)
            
        
        # add new tags
        for tag in tags:
            if tag and tag not in self.mapping:
                self.add_tag(tag)
            
        
    
    @property
    def tags(self):
        return list(self.mapping)
    
    def get_tagstring(self):
        return u' '.join(self.tags)
    
    def set_tagstring(self, tagstring):
        tags = tagstring.split(u' ')
        tags = [tag.strip() for tag in tags]
        self.update_tags(tags)
    
    tagstring = property(
        get_tagstring, 
        set_tagstring
    )




from setuptools import setup, find_packages

setup(
    name = 'largeblue.tag',
    version = '0.1',
    description = "Simple tagging adapter",
    long_description = '\n%s' % open("src/largeblue/tag/README.txt").read(),
    keywords = 'zope largeblue tag tags tagging adapter simple',
    author = 'James Arthur',
    author_email = 'firstname.lastname@largeblue.com',
    url = 'http://pesto.largeblue.net/trac/browser/largeblue/devel/largeblue.tag',
    license = 'Public Domain',
    classifiers = [
        'Programming Language :: Python',
        'Environment :: Web Environment',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Framework :: Zope3',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Natural Language :: English',
    ],
    packages=find_packages('src'),
    namespace_packages = ['largeblue'],
    package_dir = {'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'ZConfig',
        'zdaemon',
        'ZODB3',
        'zope.annotation',
        'zope.app.appsetup',
        'zope.app.authentication',
        'zope.app.catalog',
        'zope.app.tree',
        'zope.app.intid',
        'zope.app.keyreference',
        'zope.app.testing',
        'zope.app.wsgi>=3.4.0',
        'zope.app.zcmlfiles',
        'zope.app.securitypolicy',
        'zope.copypastemove',
        'zope.formlib',
        'zope.i18n',
        'zope.publisher',
        'zope.security',
        'zope.testing',
        'zope.traversing'
    ],
    entry_points = """
        [console_scripts]
        largeblue-debug = largeblue.startup:interactive_debug_prompt
        largeblue-ctl = largeblue.startup:zdaemon_controller
        [paste.app_factory]
        main = largeblue.startup:application_factory
    """
)
